@@ -1,25 +1,27 @@
 import argparse
 import os
 import sys
+from dataclasses import dataclass
+from typing import Dict, List, Union, Optional
 
 import asrp
-
-import speechmix
-from datasets import load_dataset, Audio, load_from_disk
 import torch
+import torchaudio
+from datasets import load_dataset, Audio, load_from_disk
+from torch.nn.utils.rnn import pad_sequence
 from transformers import Trainer, TrainingArguments, EarlyStoppingCallback, AutoTokenizer, TrainerCallback, \
     TrainerState, TrainerControl
-from typing import Dict, List, Union, Optional
-from dataclasses import dataclass
+
+import speechmix
 
 
 def create_self_decoder_input(decoder_model, tokenizer, input_sent, device):
-    gen_input = tokenizer(input_sent, add_special_tokens=False).input_ids
+    gen_input = tokenizer(input_sent, add_special_tokens=True).input_ids
     predicted = [decoder_model.config.decoder_start_token_id]
-    print([decoder_model.config.decoder_start_token_id])
     with torch.no_grad():
         decoder_model.eval()
-        for _ in range(decoder_model.config.max_length):
+        decoder_length = max(decoder_model.config.max_length, len(gen_input))
+        for _ in range(decoder_length):
             max_item = torch.argmax(
                 decoder_model(input_ids=torch.tensor([gen_input], device=device),
                               output_hidden_states=True,
@@ -33,23 +35,34 @@ def create_self_decoder_input(decoder_model, tokenizer, input_sent, device):
 
 
 def main(arg=None):
+    def prepare_dataset_custom(batch):
+        path = batch["path"]
+        speech, sampling_rate = torchaudio.load(path)
+        resampler = torchaudio.transforms.Resample(orig_freq=sampling_rate, new_freq=16_000)
+        batch["input_values"] = resampler.forward(speech.squeeze(0)).numpy()
+        batch["lengths"] = len(batch["input_values"])
+        batch["labels"] = model.tokenizer(batch["text"]).input_ids
+        return batch
+
     def prepare_dataset(batch, selftype=False):
         audio = batch["audio"]
         new_batch = {}
         new_batch["input_values"] = audio["array"]
-        new_batch["length"] = audio["array"].size
+        new_batch["lengths"] = audio["array"].size
         sent = batch["text"] if 'text' in batch else batch["sentence"]
         sent = sent.lower()
         if selftype:
-            decoder_input, decoder_target = model.decoder_model.eval(model.decoder_model, model.tokenizer, sent,
-                                                                     model.device)
+            input_text_prompt = "ASR: "
+            decoder_input, decoder_target = create_self_decoder_input(model.decoder_model, model.tokenizer,
+                                                                      input_text_prompt + sent,
+                                                                      model.device)
+            new_batch['input_text_prompt'] = input_text_prompt
             new_batch["text_input_ids"] = decoder_input
             new_batch['labels'] = decoder_target
         else:
             gen_input = model.tokenizer(sent, add_special_tokens=False).input_ids
             new_batch['labels'] = gen_input
         new_batch['labels'] += [model.tokenizer.eos_token_id]
-        new_batch["input_ids"] = new_batch["labels"]
         return new_batch
 
     def compute_metrics(pred):
@@ -78,9 +91,9 @@ def main(arg=None):
 
         def __call__(self, features: List[Dict[str, Union[List[int], torch.Tensor]]]) -> Dict[str, torch.Tensor]:
             batch = {}
-            batch['input_values'] = [torch.tensor(feature["input_values"]) for feature in features]
+            batch['input_values'] = pad_sequence([torch.tensor(feature["input_values"]) for feature in features],
+                                                 batch_first=True, padding_value=-100)
             label_features = [{"input_ids": feature['labels']} for feature in features]
-
             labels_batch = self.tokenizer.pad(
                 label_features,
                 padding=self.padding,
@@ -157,6 +170,7 @@ def main(arg=None):
         parser.add_argument("--HFSpeechMixAdapter", action='store_true')
         parser.add_argument("--HFSpeechMixGAN", action='store_true')
         parser.add_argument("--HFSpeechMixFixed", action='store_true')
+        parser.add_argument("--cache", action='store_true')
         parser.add_argument("--dataset", type=str)
         parser.add_argument("--field", type=str)
         parser.add_argument("--train_split", type=str)
@@ -177,6 +191,9 @@ def main(arg=None):
         parser.add_argument('--down_scale', default=8, type=int)
         parser.add_argument('--weighted_sum', action='store_true')
         parser.add_argument('--fixed_parameters', action='store_true')
+        parser.add_argument("--custom_set", type=str)
+        parser.add_argument("--max_input_length_in_sec", default=20, type=int)
+        parser.add_argument("--group_by_length", action="store_true")
         parser.add_argument('--fixed_except', nargs='+',
                             default=["layer_norm", "encoder_attn", 'enc_to_dec_proj', 'length_adapter',
                                      "layernorm_embedding", 'attention', 'encoder'])
@@ -227,25 +244,62 @@ def main(arg=None):
         model_type = "SpeechMixEED"
         model = speechmix.SpeechMixEED(**input_args)
 
-    selftype = ('SpeechMixSelf' in model_type or 'SpeechMixGAN' in model_type)
-    cache_path_train = f'./train_ds_{input_args["dataset"]}_{input_args["field"]}_{input_args["train_split"]}.parquet'
-    cache_path_valid = f'./valid_ds_{input_args["dataset"]}_{input_args["field"]}_{input_args["train_split"]}.parquet'
+    # selftype = ('SpeechMixSelf' in model_type or 'SpeechMixGAN' in model_type)
+    selftype = 'SpeechMixSelf' in model_type
+    if 'custom_set' in input_args:
+        cache_file_train = f"{input_args['custom_set']}_hf_train.data"
+        if input_args['cache'] and os.path.isdir(cache_file_train):
+            train_ds = load_dataset('csv', data_files=input_args['custom_set'])['train']
+            train_ds = train_ds.load_from_disk(cache_file_train)
+        else:
+            dataset = load_dataset('csv', data_files=input_args['custom_set'], cache_dir='./.cache')
+            dataset = dataset['train']
+            dataset = dataset.train_test_split(test_size=0.1)
+            train_ds = dataset['train']
+            train_ds = train_ds.map(prepare_dataset_custom, keep_in_memory=False, num_proc=input_args["num_proc"])
+            train_ds.save_to_disk(cache_file_train)
 
-    if os.path.exists(cache_path_train) and os.path.exists(cache_path_valid):
-        train_ds = load_from_disk(cache_path_train)
-        valid_ds = load_from_disk(cache_path_valid)
+        cache_file_test = f"{input_args['custom_set']}_hf_test.data"
+        if input_args['cache'] and os.path.isdir(cache_file_test):
+            valid_ds = load_dataset('csv', data_files=input_args['custom_set'])['train']
+            valid_ds = valid_ds.load_from_disk(cache_file_test)
+        else:
+            dataset = load_dataset('csv', data_files=input_args['custom_set'], cache_dir='./.cache')
+            dataset = dataset['train']
+            dataset = dataset.train_test_split(test_size=0.1)
+            valid_ds = dataset['test']
+            valid_ds = valid_ds.map(prepare_dataset_custom, keep_in_memory=False, num_proc=input_args["num_proc"])
+            valid_ds.save_to_disk(cache_file_test)
     else:
-        train_ds = load_dataset(input_args["dataset"], input_args["field"], split=input_args["train_split"])
-        valid_ds = load_dataset(input_args["dataset"], input_args["field"], split=input_args["test_split"])
+        cache_path_train = f'./train_ds_{input_args["dataset"]}_{model_type}_{input_args["speech_model_config"]}_{input_args["nlp_model_config"]}_{input_args["field"]}_{input_args["train_split"]}.parquet'
+        cache_path_valid = f'./valid_ds_{input_args["dataset"]}_{model_type}_{input_args["speech_model_config"]}_{input_args["nlp_model_config"]}_{input_args["field"]}_{input_args["train_split"]}.parquet'
+        if input_args['cache'] and os.path.exists(cache_path_train) and os.path.exists(cache_path_valid):
+            train_ds = load_from_disk(cache_path_train)
+            valid_ds = load_from_disk(cache_path_valid)
+        else:
+            train_ds = load_dataset(input_args["dataset"], input_args["field"], split=input_args["train_split"])
+            valid_ds = load_dataset(input_args["dataset"], input_args["field"], split=input_args["test_split"])
 
-        train_ds = train_ds.cast_column("audio", Audio(sampling_rate=16_000))
-        valid_ds = valid_ds.cast_column("audio", Audio(sampling_rate=16_000))
+            train_ds = train_ds.cast_column("audio", Audio(sampling_rate=16_000))
+            valid_ds = valid_ds.cast_column("audio", Audio(sampling_rate=16_000))
 
-        train_ds = train_ds.map(prepare_dataset, num_proc=1, fn_kwargs={"selftype": selftype})
-        valid_ds = valid_ds.map(prepare_dataset, num_proc=1, fn_kwargs={"selftype": selftype})
+            train_ds = train_ds.map(prepare_dataset, num_proc=1, fn_kwargs={"selftype": selftype})
+            valid_ds = valid_ds.map(prepare_dataset, num_proc=1, fn_kwargs={"selftype": selftype})
 
-        train_ds.save_to_disk(cache_path_train)
-        valid_ds.save_to_disk(cache_path_valid)
+            train_ds.save_to_disk(cache_path_train)
+            valid_ds.save_to_disk(cache_path_valid)
+
+    if input_args.get('max_input_length_in_sec', None):
+        max_input_length_in_sec = input_args['max_input_length_in_sec']
+        min_input_length_in_sec = 1
+        train_ds = train_ds.filter(
+            lambda
+                x: min_input_length_in_sec * 16000 < x < max_input_length_in_sec * 16000,
+            input_columns=["lengths"])
+        valid_ds = valid_ds.filter(
+            lambda
+                x: min_input_length_in_sec * 16000 < x < max_input_length_in_sec * 16000,
+            input_columns=["lengths"])
 
     data_collator = DataCollatorWithPadding(tokenizer=model.tokenizer, padding=True,
                                             selftype=selftype)
@@ -256,11 +310,11 @@ def main(arg=None):
         per_device_eval_batch_size=int(input_args['batch']),
         gradient_accumulation_steps=int(input_args['grad_accum']),
         eval_accumulation_steps=2,
+        group_by_length=input_args["group_by_length"],
         optim="adafactor",
         evaluation_strategy="steps",
-        group_by_length=True,
-        fp16=input_args.get('fp16', True),
         load_best_model_at_end=True,
+        fp16=input_args.get('fp16', True),
         num_train_epochs=input_args.get('epoch', 10),
         save_steps=input_args.get('eval_step', 700),
         eval_steps=input_args.get('eval_step', 700),
